<?php
/**
 * @file
 * Drush integration for cp2p2.
 */

/**
 * Implements hook_drush_command().
 */
function cp2p2_drush_command() {
  $items['cp2p2-status'] = array(
    'callback' => 'drush_cp2p2_status',
    'description' => dt('Profile a status of the CP2P2 conversion.'),
    'aliases' => array('cp2p2'),
    'drupal dependencies' => array('profile2', 'cp2p2'),
  );
  $items['cp2p2-set-type'] = array(
    'callback' => 'drush_cp2p2_set_type',
    'description' => 'Assign the profile type name to be used for a given content type. Once the structure is converted this cannot be re-done.',
    'drupal dependencies' => array('profile2', 'cp2p2'),
    'arguments' => array(
      'old' => 'Existing content type name.',
      'new' => 'Profile type name that will be used for the conversion.',
    ),
  );
  $items['cp2p2-convert'] = array(
    'callback' => 'drush_cp2p2_convert',
    'description' => 'Convert a Content Profile type to a Profile2 profile type.',
    'drupal dependencies' => array('profile2', 'cp2p2'),
    'arguments' => array(
      'type' => 'Name of the content type to be converted.',
    ),
  );
  $items['cp2p2-migrate'] = array(
    'callback' => 'drush_cp2p2_migrate',
    'description' => 'Move data from a Content Profile type to a Profile2 profile type.',
    'drupal dependencies' => array('profile2', 'cp2p2'),
    'arguments' => array(
      'type' => 'Name of the content type to be converted.',
    ),
  );
  $items['cp2p2-cleanup'] = array(
    'callback' => 'drush_cp2p2_cleanup',
    'description' => dt('Remove remaining Content Profile settings after they are converted.'),
    'drupal dependencies' => array('profile2', 'cp2p2'),
  );
  $items['cp2p2-aio'] = array(
    'callback' => 'drush_cp2p2_aio',
    'description' => dt('Run the complete migration in one command.'),
    'drupal dependencies' => array('profile2', 'cp2p2'),
  );
  $items['cp2p2-purge-all'] = array(
    'callback' => 'drush_cp2p2_purge_all',
    'description' => dt('Remove all converted profile data.'),
    'drupal dependencies' => array('profile2', 'cp2p2'),
  );
  return $items;
}

/**
 * Provide a status for the migration.
 */
function drush_cp2p2_status() {
  // Get a list of all Content Profile content types.
  $content_types = _cp2p_content_types_list();

  // Get a list of all Profile2 profile types.
  $profile_types = profile2_get_types();

  // Compile a status table.
  $statuses = array(
    array(
      dt('Content type'),
      '',
      dt('Profile type'),
      '',
      dt('Structure'),
      '',
      dt('Data'),
    ),
  );
  foreach ($content_types as $content_type_name) {
    $profile_type_name = variable_get('cp2p_type_' . $content_type_name, $content_type_name);

    if (variable_get('cp2p_converted_' . $content_type_name)) {
      $structure_status = dt('Converted');
    }
    elseif (isset($profile_types[$profile_type_name])) {
      $structure_status = dt('Exists');
    }
    else {
      $structure_status = dt('Not converted');
    }
    if (variable_get('cp2p_data_' . $content_type_name)) {
      $data_status = dt('Migrated');
    }
    else {
      $data_status = dt('Not migrated');
    }

    $statuses[] = array(
      $content_type_name,
      ':',
      $profile_type_name,
      ':',
      $structure_status,
      ':',
      $data_status,
    );
  }

  drush_print_table($statuses, TRUE);
  drush_print(dt('Use the "cp2p2-convert" command to first convert content types to a specific profile type and then the command "cp2p2-migrate" to migrate the data; the "cp2p2-set-type" command may be used to override the destination profile type.'));
}

/**
 * Override the profile type.
 */
function drush_cp2p2_set_type($old, $new) {
  $content_types = node_type_get_types();

  // Ensure the requested content type actually exists.
  if (!isset($content_types[$old])) {
    return drush_set_error('CONTENT_TYPE_DOESNT_EXIST', dt('The requested content type, "!type", doesn\'t exist, please confirm the name via the "cp2p2-status" command.', array('!type' => $old)));
  }

  // If the old & new values are the same, remove the variable as that is the
  // default.
  if ($old == $new) {
    variable_del('cp2p_type_' . $old);
  }
  else {
    variable_set('cp2p_type_' . $old, $new);
  }

  drush_print(dt('The "!old" content type will be converted to the "!new" profile type.', array('!old' => $old, '!new' => $new)));
}

/**
 * Convert a content type to a profile type.
 */
function drush_cp2p2_convert($content_type_name) {
  $content_types = node_type_get_types();
  $all_fields = field_read_fields();

  // Ensure the requested content type actually exists.
  if (!isset($content_types[$content_type_name])) {
    return drush_set_error('CONTENT_TYPE_DOESNT_EXIST', dt('The requested content type, "!type", doesn\'t exist, please confirm the name via the "cp2p2-status" command.', array('!type' => $content_type_name)));
  }

  $content_type = $content_types[$content_type_name];

  // Identify the destination profile type.
  $profile_type_name = variable_get('cp2p_type_' . $content_type_name, $content_type_name);

  // Get a list of all Profile2 profile types.
  $profile_types = profile2_get_types();

  // Can't re-create the same profile type.
  if (isset($profile_types[$profile_type_name])) {
    return drush_set_error('PROFILE_TYPE_ALREADY_EXISTS', dt('The requested profile type, "!type", already exists and will not be replaced.', array('!type' => $profile_type_name)));
  }

  // Step 1: Create the profile type.
  $params = array(
    'userCategory' => TRUE,
    'userView' => TRUE,
    'type' => $profile_type_name,
    'label' => $content_type->name,
    'weight' => 0,
    'is_new' => TRUE,
  );
  $profile_type = new ProfileType($params);
  profile2_type_save($profile_type);

  // Step 2: Attach the fields to the profile type.
  $params = array(
    'entity_type' => 'node',
    'bundle' => $content_type_name,
  );
  foreach (field_read_instances($params) as $ctr => $field) {
    $field['entity_type'] = 'profile2';
    $field['bundle'] = $profile_type_name;

    // The main 'body' field can't be converted, it's a node-only thing, so
    // create a replacement.
    if ($field['field_name'] == 'body') {
      // The new field's name will have the profile type name as a prefix.
      $field['field_name'] = "field_{$profile_type_name}_{$field['field_name']}";

      // If the field doesn't already exist, create it.
      if (!isset($all_fields[$field['field_name']])) {
        // The new field needs to work the same as the old one.
        $field['type'] = 'text_with_summary';

        // Remove the IDs.
        unset($field['id']);
        unset($field['field_id']);
        unset($field['deleted']);

        // Create the new field.
        field_create_field($field);
      }
    }

    // Save the instance. This seems safe to do for fields just created with
    // field_create_field().
    field_create_instance($field);
  }

  // All done!
  variable_set('cp2p_converted_' . $content_type_name, TRUE);
  drush_print(dt('A new "!profile_type_name" profile type has be created that matches the old "!content_type_name" content type.', array('!content_type_name' => $content_type_name, '!profile_type_name' => $profile_type_name)));
}

/**
 * TODO: Migrate the data from the old fields.
 */
function drush_cp2p2_migrate($content_type_name) {
  $content_types = node_type_get_types();

  // Ensure the requested content type actually exists.
  if (!isset($content_types[$content_type_name])) {
    return drush_set_error('CONTENT_TYPE_DOESNT_EXIST', dt('The requested content type, "!type", doesn\'t exist, please confirm the name via the "cp2p2-status" command.', array('!type' => $content_type_name)));
  }

  // Identify the destination profile type.
  $profile_type_name = variable_get('cp2p_type_' . $content_type_name, $content_type_name);

  // Load all profile type definitions.
  $profile_types = profile2_get_types();

  // Ensure the requested profile type actually exists.
  if (!isset($profile_types[$profile_type_name])) {
    return drush_set_error('PROFILE_TYPE_DOESNT_EXIST', dt('The requested profile type, "!type", doesn\'t exist, please confirm via the "cp2p2-status" command that the content type has been converted.', array('!type' => $profile_type_name)));
  }

  // Load all field specs.
  $params = array(
    'entity_type' => 'profile2',
    'bundle' => $profile_type_name,
  );
  $fields = array();
  foreach (field_read_instances($params) as $ctr => $field) {
    $fields[$field['field_name']] = $field;
  }

  // Loop over each node.
  $records = db_query("SELECT nid FROM {node} WHERE type=:type", array(':type' => $content_type_name));

  foreach ($records as $record) {
    // Load the profile node.
    $node = node_load($record->nid);

    // Build the new profile object.
    $profile = array();
    $profile['type'] = $profile_type_name;
    $profile['uid'] = $node->uid;
    $profile['created'] = $node->created;

    // Copy each field.
    foreach (get_object_vars($node) as $key => $val) {
      // Only copy items with a name starting in 'field_'.
      if (strpos($key, 'field_') === 0) {
        // Optional alter the field data.
        $function = '_cp2p2_' . $fields[$key]['widget_type'] . '_alter';
        if (function_exists($function)) {
          $function($val);
        }
        $profile[$key] = $val;
      }
    }
    // Copy the body field.
    if (!empty($node->body)) {
      $field_name = "field_{$profile_type_name}_body";
      $profile[$field_name] = array(
        LANGUAGE_NONE => array(
          array(
            'value' => $node->body,
            'format' => $node->format,
          ),
        ),
      );
    }

    // Save the profile record.
    $new_profile = profile2_create($profile);
    $new_profile->save();

    variable_set('cp2p_data_' . $content_type_name, TRUE);
    drush_print(dt('Converted the !profile_type_name for !user.', array('!profile_type_name' => $profile_type_name, '!user' => $node->name)));
  }
}

/**
 * Cleanup the current installation by removing the old content types.
 */
function drush_cp2p2_cleanup() {
  if (!drush_confirm(dt('Do you wish to remove all Content Profile content types, nodes and settings?'))) {
    return drush_user_abort();
  }

  foreach (node_type_get_types() as $type_name => $type_data) {
    // Delete the content type.
    if (variable_get("content_profile_use_{$type_name}", FALSE)) {
      node_type_delete($type_name);
      variable_del('node_preview_' . $type_name);
      watchdog('node', 'Deleted content type %name.', array('%name' => $type_name), WATCHDOG_NOTICE);
      node_types_rebuild();
    }

    // Delete the two possible variables.
    variable_del("content_profile_{$type_name}");
    variable_del("content_profile_use_{$type_name}");
  }
  menu_rebuild();
}

/**
 * Run all of the necessary steps thus migrating all fields in one go.
 */
function drush_cp2p2_aio() {
  foreach (_cp2p_content_types_list() as $type) {
    drush_cp2p2_convert($type);
    drush_cp2p2_migrate($type);
  }

  drush_print(dt('All done.'));
}

function drush_cp2p2_purge_all() {
  if (!drush_confirm(dt('Do you wish to remove all converted user profiles and settings?'))) {
    return drush_user_abort();
  }

  // Load all profile types.
  $profile_types = entity_load_multiple_by_name('profile2_type', FALSE);

  // Remove all user profiles.
  foreach (_cp2p_content_types_list() as $content_type_name) {
    // Work out the profile type being used.
    $profile_type_name = variable_get('cp2p_type_' . $content_type_name, $content_type_name);

    // Get a list of all profile IDs.
    $ids = db_query("SELECT pid FROM {profile} WHERE type=:type", array(':type' => $profile_type_name))->fetchCol();
    drush_print(dt('Purging all !profile_type_name records...', array('!profile_type_name' => $profile_type_name)));

    // Delete the data.
    profile2_delete_multiple($ids);
    drush_print(dt('All !profile_type_name records purged.', array('!profile_type_name' => $profile_type_name)));

    // Remove the profile type itself.
    $profile_types[$profile_type_name]->delete();
    drush_print(dt('The "!profile_type_name" profile type has been removed.', array('!profile_type_name' => $profile_type_name)));

    // Remove any extra variables.
    variable_del('cp2p_converted_' . $content_type_name);
    variable_del('cp2p_data_' . $content_type_name);
    variable_del('cp2p_type_' . $content_type_name);
    drush_print(dt('The "!profile_type_name" CP2P2 settings have been removed.', array('!profile_type_name' => $profile_type_name)));
  }

  drush_print(dt('All done.'));
}

/**
 * Get a list of all content types that have been enabled for use with the
 * Content Profile module.
 *
 * @return
 *   Simple array ($type=>$type) of all supported content types.
 */
function _cp2p_content_types_list() {
  $content_types = &drupal_static(__FUNCTION__);

  // If the list wasn't compiled before, do so now.
  if (!isset($content_types)) {
    // Check each content type in turn.
    foreach (node_type_get_types() as $type_name => $type_data) {
      // Content Profile used a variable in format "content_profile_use_[type]"
      // to control whether the content type is used.
      if (variable_get("content_profile_use_{$type_name}", FALSE)) {
        $content_types[$type_name] = $type_name;
      }
    }
  }

  return $content_types;
}

/**
 * Callback for altering a location field prior to saving.
 */
function _cp2p2_location_alter(&$val) {
  // Remove the 'lid' value so that it will be saved as a new record.
  if (isset($val[LANGUAGE_NONE][0]['lid'])) {
    unset($val[LANGUAGE_NONE][0]['lid']);
  }
}
